@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -52,16 +52,13 @@ void xmms_cfg_dummy(ConfigDb *dummy) { }
 
 
 /* init with DEFAULT_CFG to make sure all string pointers are set to NULL */
-static config_t _cfg = CONFIG_DEFAULT;
+config_t _cfg = CONFIG_DEFAULT;  /* also used in configure.c */
 static config_t *cfg = &_cfg;
 
-/* from crossfade.c */
-extern pthread_mutex_t buffer_mutex;
-
 /*****************************************************************************/
 
 static void
-update_plugin_config(gchar ** config_string, gchar *name, plugin_config_t *pc, gboolean save);
+update_plugin_config(gchar **config_string, gchar *name, plugin_config_t *pc, gboolean save);
 
 static void
 g_free_f(gpointer data, gpointer user_data)
