@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -143,8 +143,8 @@ static GtkWidget *about_win = NULL;
 static GtkWidget *set_wgt;
 static GtkWidget *get_wgt;
 
-/* init with DEFAULT_CFG to make sure all string pointers are set to NULL */
-static config_t _cfg = CONFIG_DEFAULT;
+/* defined in cfgutil.c */
+extern config_t _cfg;
 static config_t *cfg = &_cfg;
 
 /* some helpers to keep track of the GUI's state */
@@ -154,7 +154,7 @@ static plugin_config_t op_config;
 static gint ep_index;
 
 /* from crossfade.c */
-extern pthread_mutex_t buffer_mutex;
+extern MUTEX buffer_mutex;
 
 /*** internal helpers ********************************************************/
 
@@ -256,7 +256,6 @@ scan_devices(gchar *type, GtkWidget *option_menu, activate_func_t signal_f)
 
 	GtkWidget *menu;
 	gint index = 0;
-	gint mixer = 0;
 
 	menu = gtk_menu_new();
 
@@ -317,6 +316,7 @@ scan_devices(gchar *type, GtkWidget *option_menu, activate_func_t signal_f)
 		{
 			gchar dev_name[32];
 			int fd;
+			gint mixer = 0;
 			mixer_info info;
 			gchar *label;
 
@@ -1412,7 +1412,7 @@ on_config_apply_clicked(GtkButton *button, gpointer user_data)
 	/* presets */
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* free existing strings */
 	if (config->oss_alt_audio_device) g_free(config->oss_alt_audio_device);
@@ -1433,7 +1433,7 @@ on_config_apply_clicked(GtkButton *button, gpointer user_data)
 	xfade_realize_config();
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* save configuration */
 	xfade_save_config();
@@ -1737,7 +1737,7 @@ xfade_about()
 	{
 		gchar *about_text =
 			"XMMS Crossfade Plugin " VERSION "\n"
-			"Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>\n"
+			"Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>\n"
 			"\n"
 			"based on the original OSS Output Plugin  Copyright (C) 1998-2000\n"
 			"Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies\n"
