@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -73,7 +73,6 @@ void xfade_stop_monitor() { }
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
-
 #undef  DEBUG_HARDCORE
 
 /* output plugin callback prototypes */
@@ -152,13 +151,24 @@ static gboolean *xmms_is_quitting             = NULL;   /* XMMS */
 static gboolean *input_stopped_for_restart    = NULL;   /* XMMS */
 static char *  (*playlist_get_fadeinfo)(int)  = NULL;   /* XMMS patch */
 
+static void (*xmms_input_get_song_info)(gchar *, gchar **, gint *); /* XMMS */
+static gchar **xmms_gentitle_format = NULL;   /* XMMS private cfg */
+
 #if defined(COMPILE_FOR_BMP) || defined(COMPILE_FOR_AUDACIOUS)
-gboolean bmp_playback_get_playing();  /* BMP */
+#if defined(AUDACIOUS_NEW_ABI)
+gboolean playback_get_playing();  /* >= Audacious-1.3.0 */
+gboolean get_input_playing()
+{
+	return playback_get_playing();
+}
+#else
+gboolean bmp_playback_get_playing();  /* Audacious, BMP */
 gboolean get_input_playing()
 {
 	return bmp_playback_get_playing();
 }
 #endif
+#endif
 
 #ifdef HAVE_CTOR
 void fini() __attribute__ ((destructor));
@@ -270,8 +280,8 @@ static format_t out_format;
 static buffer_t            the_buffer;
        buffer_t *buffer = &the_buffer;
 
-       pthread_mutex_t  buffer_mutex;
-static pthread_t        buffer_thread;
+static THREAD buffer_thread;
+       MUTEX  buffer_mutex = MUTEX_INITIALIZER;
 
 static effect_context_t  effect_context;
 static convert_context_t convert_context;
@@ -359,7 +369,7 @@ xfade_realize_config()		/* also called by xfade_init() */
 		/* 0.3.1: HACK: Somehow, if we open output here at XMMS startup, there
 		   will be leftover filedescriptors later when closing output again.
 		   Opening output in a timeout function seems to work around this... */
-		DEBUG(("[crossfade] realize_config: adding timeout (pid=%d)\n", getpid()));
+		DEBUG(("[crossfade] realize_config: adding timeout (pid=%d)\n", (int) getpid()));
 		g_timeout_add(0, open_output_f, NULL);
 	}
 }
@@ -512,9 +522,14 @@ open_output()
 	stopped = FALSE;
 
 	/* create and run buffer thread */
+#if defined(HAVE_GLIB_STATIC_MUTEX)
+	buffer_thread = g_thread_create((GThreadFunc) buffer_thread_f, NULL, TRUE, NULL);
+	if (buffer_thread == NULL)
+#else
 	if (pthread_create(&buffer_thread, NULL, buffer_thread_f, NULL))
+#endif
 	{
-		PERROR("[crossfade] open_output: pthread_create()");
+		PERROR("[crossfade] open_output: thread_create()");
 		g_free(buffer->data);
 		the_op->close_audio();
 		the_op = NULL;
@@ -552,9 +567,6 @@ xfade_init()
 	/* show monitor win if enabled in config */
 	xfade_check_monitor_win();
 
-	/* create buffer mutex (never fails) */
-	pthread_mutex_init(&buffer_mutex, NULL);
-
 	/* init contexts */
 	effect_init (&effect_context, NULL);
 	convert_init(&convert_context);
@@ -591,6 +603,8 @@ load_symbols()
 #ifdef HAVE_DLFCN_H
 	void *handle;
 	char *error;
+	gchar **xmms_cfg;
+	gchar * (*get_gentitle_format)();
 
 	/* open ourselves (that is, the XMMS binary) */
 	handle = dlopen(NULL, RTLD_NOW);
@@ -605,10 +619,6 @@ load_symbols()
 	input_stopped_for_restart = dlsym(handle, "input_stopped_for_restart");
 	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
 
-	/* DEBUG(("[crossfade] load_symbols: playlist_get_info_going:")); */
-	xmms_playlist_get_info_going = dlsym(handle, "playlist_get_info_going");
-	/* DEBUG((!(error = dlerror())? " found\n" : " missing\n")); */
-
 	DEBUG(("[crossfade] load_symbols: is_quitting:"));
 	xmms_is_quitting = dlsym(handle, "is_quitting");
 	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
@@ -617,6 +627,24 @@ load_symbols()
 	playlist_get_fadeinfo = dlsym(handle, "playlist_get_fadeinfo");
 	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
 
+	/* check for some XMMS functions */
+	xmms_playlist_get_info_going = dlsym(handle, "playlist_get_info_going");
+	xmms_input_get_song_info     = dlsym(handle, "input_get_song_info");
+
+	/* HACK: direct access to XMMS' config 'gentitle_format' */
+	xmms_cfg = dlsym(handle, "cfg");
+	get_gentitle_format = dlsym(handle, "xmms_get_gentitle_format");
+	if (xmms_cfg && get_gentitle_format)
+	{
+		gchar *format = get_gentitle_format();
+
+		int i = 128;
+		gchar **p = (gchar **)xmms_cfg;
+		for (i = 128; i > 0 && *p != format; i--, p++);
+		if (*p == format)
+			xmms_gentitle_format = p;
+	}
+
 	dlclose(handle);
 #endif
 }
@@ -650,7 +678,7 @@ fini()
 	DEBUG(("[crossfade] fini: cleanup:\n"));
 
 	/* wait for buffer_thread to stop */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/*
 	 * Special handling if the output is still opened. This can only happen
@@ -677,9 +705,9 @@ fini()
 	DEBUG(("[crossfade] fini: cleanup: waiting for buffer thread...\n"));
 	while (output_opened)  /* output_opened) */
 	{
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		xmms_usleep(10000);
-		pthread_mutex_lock(&buffer_mutex);
+		MUTEX_LOCK(&buffer_mutex);
 	}
 	DEBUG(("[crossfade] fini: cleanup: waiting for buffer thread... done\n"));
 	
@@ -687,7 +715,7 @@ fini()
 	/*
 	 * HACK: 0.3.9: DO NOT close the output plugin from within fini().
 	 *
-	 * For yet unexplained reasons, pthread_join() will not work from
+	 * For yet unexplained reasons, thread_join() will not work from
 	 * within code executed by a dynamic library's destructor code.
 	 *
 	 * http://groups.google.de/group/comp.programming.threads/msg/17b4c7a4f35bd1c5
@@ -706,8 +734,7 @@ fini()
 	DEBUG(("[crossfade] fini: cleanup: done\n"));
 
 	/* unlock and free buffer mutex */
-	pthread_mutex_unlock(&buffer_mutex);
-	pthread_mutex_destroy(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* free contexts */
 #ifdef HAVE_LIBFFTW
@@ -956,8 +983,8 @@ xfade_apply_fade_config(fade_config_t *fc)
 		for (n = blen / 4; n > 0; n--)
 		{
 			gfloat factor = 1.0f - (((gfloat) fade / out_len) * out_scale);
-			*p = (gfloat) * p * factor; p++;
-			*p = (gfloat) * p * factor; p++;
+			*p = (gfloat)*p * factor; p++;
+			*p = (gfloat)*p * factor; p++;
 			fade += 4;
 		}
 
@@ -1018,11 +1045,17 @@ xfade_apply_fade_config(fade_config_t *fc)
 static gint
 extract_track(const gchar *name)
 {
+#if 1
+	/* skip non-digits at beginning */
+	while (*name && !isdigit(*name))
+		name++;
+		
+	return atoi(name);
+#else
 	/* Remove all but numbers.
 	 * Will not work if a filename has number in the title, like "track-03-U2.mp3" 
 	 * Ideally, should look into id3 track entry and fallback to filename
 	 * */
-
 	gchar temp[8];
 	int t = 0;
 
@@ -1038,6 +1071,7 @@ extract_track(const gchar *name)
 		name++;
 	}
 	return atoi(temp);
+#endif
 }
 
 static gint
@@ -1088,17 +1122,15 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 {
 	gint pos;
 	gchar *file, *title, *comment;
-	/* id3_t id3; */
+#if defined(HAVE_ID3LIB)
+	id3_t id3;
+#endif
 
 	struct timeval tv;
 	glong dt;
 
 	DEBUG(("[crossfade]\n"));
-	DEBUG(("[crossfade] open_audio: XMMS-crossfade " VERSION "\n"));
-	
-#if 0
-	DEBUG(("[crossfade] open_audio: pid=%d\n", getpid()));
-#endif
+	DEBUG(("[crossfade] open_audio: XMMS-crossfade " VERSION ", pid=%d\n", (int) getpid()));
 
 	/* sanity... don't do anything about it */
 	if (opened)
@@ -1119,13 +1151,39 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 	DEBUG(("[crossfade] open_audio: bname=\"%s\"\n", g_basename(file)));
 	DEBUG(("[crossfade] open_audio: title=\"%s\"\n", title));
 
-	/* try to read comment from ID3 tag */
 #if 0
+	/* HACK: try to get comment and track number from xmms by sneaking in a custom title format */
+	if (xmms_gentitle_format)
+	{
+		gchar *old_gentitle_format = *xmms_gentitle_format;
+
+		gchar *temp_title  = NULL;
+		gint   temp_length = 0;
+
+		xmms_input_get_song_info(file, &temp_title, &temp_length);
+		DEBUG(("[crossfade] open_audio: TITLE: %s\n", temp_title));
+		g_free(temp_title);
+		
+		*xmms_gentitle_format = "%n/%c";
+
+		xmms_input_get_song_info(file, &temp_title, &temp_length);
+		DEBUG(("[crossfade] open_audio: TRACK/COMMENT: %s\n", temp_title));
+		g_free(temp_title);
+
+		*xmms_gentitle_format = old_gentitle_format;
+	}
+#endif
+
+	/* try to read comment from ID3 tag */
+#if defined(HAVE_ID3LIB)
 	if (!comment && get_id3(file, &id3))
 		comment = g_strdup(id3.comment);
 #endif
 
-	DEBUG(("[crossfade] open_audio: comment=\"%s\"\n", comment));
+	if (comment)
+		DEBUG(("[crossfade] open_audio: comment=\"%s\"\n", comment))
+	else
+		DEBUG(("[crossfade] open_audio: comment=NULL\n"));
 
 	/* is this an automatic crossfade? */
 	if (last_filename && (fade_config == &config->fc[FADE_CONFIG_XFADE]))
@@ -1180,7 +1238,7 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 	if (fade_config == &config->fc[FADE_CONFIG_XFADE])
 	{
 		DEBUG(("*** XFADE:\n"));
-		int current_length = playlist_get_current_length();
+		int current_length = PLAYLIST_GET_CURRENT_LENGTH;
 		DEBUG(("*** length=%d\n", current_length));
 		if (current_length < 30 * 1000)
 			fade_config = &config->fc[FADE_CONFIG_ALBUM];
@@ -1326,7 +1384,7 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 		is_http = FALSE;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* reset writer timeout */
 	gettimeofday(&last_write, NULL);
@@ -1355,7 +1413,7 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 		if (open_output())
 		{
 			DEBUG(("[crossfade] open_audio: error opening/configuring output!\n"));
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			return 0;
 		}
 		fade_config = &config->fc[FADE_CONFIG_START];
@@ -1452,7 +1510,7 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 	output_offset = the_op->written_time() + B2MS(buffer->used) + B2MS(buffer->silence_len);
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* done */
 	return 1;
@@ -1502,7 +1560,7 @@ xfade_write_audio(void *ptr, int length)
 #endif
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* check if device has been closed, reopen if necessary */
 	if (!output_opened)
@@ -1510,7 +1568,7 @@ xfade_write_audio(void *ptr, int length)
 		if (open_output())
 		{
 			DEBUG(("[crossfade] write_audio: reopening failed!\n"));
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			return;
 		}
 	}
@@ -1647,8 +1705,8 @@ xfade_write_audio(void *ptr, int length)
 		for (n = blen / 4; n > 0; n--)
 		{
 			gfloat factor = 1.0f - (((gfloat) buffer->fade / buffer->fade_len) * buffer->fade_scale);
-			*p = (gfloat) * p * factor; p++;
-			*p = (gfloat) * p * factor; p++;
+			*p = (gfloat)*p * factor; p++;
+			*p = (gfloat)*p * factor; p++;
 			buffer->fade -= 4;
 		}
 	}
@@ -1667,8 +1725,8 @@ xfade_write_audio(void *ptr, int length)
 
 		for (n = blen / 2; n > 0; n--)
 		{
-			gint out = (gint) * p1 + *p2++;  /* add */
-			if (out > 32767)                 /* clamp */
+			gint out = (gint)*p1 + *p2++;  /* add */
+			if (out > 32767)               /* clamp */
 				*p1++ = 32767;
 			else if (out < -32768)
 				*p1++ = -32768;
@@ -1699,7 +1757,7 @@ xfade_write_audio(void *ptr, int length)
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 #ifdef DEBUG_HARDCORE
 	DEBUG(("[crossfade] write_audio: done.\n"));
 #endif
@@ -1751,9 +1809,9 @@ sync_output()
 		}
 
 		/* yield */
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		xmms_usleep(10000);
-		pthread_mutex_lock(&buffer_mutex);
+		MUTEX_LOCK(&buffer_mutex);
 	}
 
 	/* calculate total time we spent in here */
@@ -1789,7 +1847,7 @@ buffer_thread_f(void *arg)
 	DEBUG(("[crossfade] buffer_thread_f: thread started (pid=%d)\n", (int) getpid()));
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	while (!stopped)
 	{
@@ -1797,9 +1855,9 @@ buffer_thread_f(void *arg)
 #ifdef DEBUG_HARDCORE
 		DEBUG(("[crossfade] buffer_thread_f: yielding...\n"));
 #endif
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		xmms_usleep(10000);
-		pthread_mutex_lock(&buffer_mutex);
+		MUTEX_LOCK(&buffer_mutex);
 
 		/* --------------------------------------------------------------------- */
 
@@ -2178,8 +2236,8 @@ buffer_thread_f(void *arg)
 					DEBUG(("[crossfade] buffer_thread_f: reopening output plugin failed!\n"));
 					g_free(buffer->data);
 					output_opened = FALSE;
-					pthread_mutex_unlock(&buffer_mutex);
-					pthread_exit(0);
+					MUTEX_UNLOCK(&buffer_mutex);
+					THREAD_EXIT(0);
 					return NULL;
 				}
 
@@ -2245,11 +2303,11 @@ buffer_thread_f(void *arg)
 	/* ----------------------------------------------------------------------- */
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* done */
 	DEBUG(("[crossfade] buffer_thread_f: thread finished\n"));
-	pthread_exit(0);
+	THREAD_EXIT(0);
 	return NULL;
 }
 
@@ -2259,13 +2317,13 @@ xfade_close_audio()
 	DEBUG(("[crossfade] close:\n"));
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* sanity... the vorbis plugin likes to call close_audio() twice */
 	if (!opened)
 	{
 		DEBUG(("[crossfade] close: WARNING: not opened!\n"));
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		return;
 	}
 
@@ -2322,16 +2380,16 @@ xfade_close_audio()
 			/* wait for buffer thread to clean up and terminate */
 			stopped = TRUE;
 #if 1
-			pthread_mutex_unlock(&buffer_mutex);
-			if (pthread_join(buffer_thread, NULL))
+			MUTEX_UNLOCK(&buffer_mutex);
+			if (THREAD_JOIN(buffer_thread))
 				PERROR("[crossfade] close: phtread_join()");
-			pthread_mutex_lock(&buffer_mutex);
+			MUTEX_LOCK(&buffer_mutex);
 #else				
 			while (output_opened)
 			{
-				pthread_mutex_unlock(&buffer_mutex);
+				MUTEX_UNLOCK(&buffer_mutex);
 				xmms_usleep(10000);
-				pthread_mutex_lock(&buffer_mutex);
+				MUTEX_LOCK(&buffer_mutex);
 			}
 #endif
 		}
@@ -2429,7 +2487,7 @@ xfade_close_audio()
 	input_playing = FALSE;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 }
 
 void
@@ -2462,7 +2520,7 @@ xfade_flush(gint time)
 #endif
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* update streampos with new stream position (input format size) */
 	streampos = ((gint64) time * in_format.bps / 1000) & -4;
@@ -2504,7 +2562,7 @@ xfade_flush(gint time)
 	output_offset = the_op->written_time() - time + B2MS(buffer->used) + B2MS(buffer->silence_len);
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 #ifdef DEBUG_HARDCORE
 	DEBUG(("[crossfade] flush: time=%d: done.\n", time));
@@ -2515,7 +2573,7 @@ void
 xfade_pause(short p)
 {
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	if (p)
 	{
@@ -2551,8 +2609,8 @@ xfade_pause(short p)
 				for (n = blen / 4; n > 0; n--)
 				{
 					gfloat factor = 1.0f - ((gfloat) fade / out_len);
-					*p = (gfloat) * p * factor; p++;
-					*p = (gfloat) * p * factor; p++;
+					*p = (gfloat)*p * factor; p++;
+					*p = (gfloat)*p * factor; p++;
 					fade += 4;
 				}
 
@@ -2573,8 +2631,8 @@ xfade_pause(short p)
 				for (n = blen / 4; n > 0; n--)
 				{
 					gfloat factor = (gfloat) fade / in_len;
-					*p = (gfloat) * p * factor; p++;
-					*p = (gfloat) * p * factor; p++;
+					*p = (gfloat)*p * factor; p++;
+					*p = (gfloat)*p * factor; p++;
 					fade += 4;
 				}
 
@@ -2604,7 +2662,7 @@ xfade_pause(short p)
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 }
 
 gint
@@ -2624,7 +2682,7 @@ xfade_buffer_free()
 	}
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	size = buffer->size;
 
@@ -2654,7 +2712,7 @@ xfade_buffer_free()
 		free = 0;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* Convert to input format size. For input rates > output rate this will
 	 * return less free space than actually is available, but we don't care. */
@@ -2687,8 +2745,11 @@ xfade_buffer_playing()
 	if (paused)
 		playing = TRUE;
 	else
-		playing = (is_http && (buffer->used > 0) && the_op->buffer_playing())
-			|| ((buffer->reopen >= 0) || (buffer->silence > 0) || (buffer->silence_len > 0));
+		playing =
+			(is_http && (buffer->used > 0) && the_op->buffer_playing())
+			|| (buffer->reopen >= 0)
+			|| (buffer->silence > 0)
+			|| (buffer->silence_len > 0);
 
 #ifdef DEBUG_HARDCORE
 	DEBUG(("[crossfade] buffer_playing: %d\n", playing));
@@ -2718,14 +2779,14 @@ xfade_output_time()
 		return 0;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	time = the_op->output_time() - output_offset;
 	if (time < 0)
 		time = 0;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 #ifdef DEBUG_HARDCORE
 	DEBUG(("[crossfade] output_time: time=%d\n", time));
@@ -2739,7 +2800,7 @@ xfade_cleanup()
 	DEBUG(("[crossfade] cleanup:\n"));
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 	
 	/* check if buffer thread is still running */
 	if (output_opened)
@@ -2749,14 +2810,14 @@ xfade_cleanup()
 		stopped = TRUE;
 		
 		/* wait for buffer thread to clean up and terminate */
-		pthread_mutex_unlock(&buffer_mutex);
-		if (pthread_join(buffer_thread, NULL))
-			PERROR("[crossfade] cleanup: phtread_join()");
-		pthread_mutex_lock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
+		if (THREAD_JOIN(buffer_thread))
+			PERROR("[crossfade] close: thread_join()");
+		MUTEX_LOCK(&buffer_mutex);
 	}
 	
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 		
 	DEBUG(("[crossfade] cleanup: done\n"));
 }
