@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -135,7 +135,7 @@
 #define DEFAULT_OSS_ALT_AUDIO_DEVICE "/dev/dsp"
 #define DEFAULT_OSS_ALT_MIXER_DEVICE "/dev/mixer"
 #define DEFAULT_OP_CONFIG_STRING     "libOSS.so=0,1,2304,0; libdisk_writer.so=1,0,2304,1"
-#define DEFAULT_OP_NAME              "libOSS.so"
+#define DEFAULT_OP_NAME              "libALSA.so"
 #define DEFAULT_EP_NAME              "libnormvol.so"
 
 #define DEFAULT_OP_CONFIG			\
@@ -362,7 +362,14 @@ gboolean get_input_playing();
 GList *get_output_list();
 GList *get_effect_list();
 gint get_playlist_position();
+#if defined(AUDACIOUS_NEW_ABI)
+struct Playlist *playlist_get_active();
+gint playlist_get_current_length(struct Playlist *);
+#define PLAYLIST_GET_CURRENT_LENGTH playlist_get_current_length(playlist_get_active())
+#else
 gint playlist_get_current_length();
+#define PLAYLIST_GET_CURRENT_LENGTH playlist_get_current_length()
+#endif
 #ifdef HAVE_PLAYLIST_GET_FILENAME
 gchar *playlist_get_filename(gint pos);
 #endif
@@ -557,9 +564,27 @@ config_t;
 
 /* some global vars... we should really get rid of those somehow */
 extern config_t *config;
-extern config_t config_default;
+extern config_t  config_default;
 extern buffer_t *buffer;
 
+#if defined(HAVE_GLIB_STATIC_MUTEX)
+#  define MUTEX GStaticMutex
+#  define MUTEX_LOCK(mutex)   g_static_mutex_lock(mutex)
+#  define MUTEX_UNLOCK(mutex) g_static_mutex_unlock(mutex)
+#  define MUTEX_INITIALIZER   G_STATIC_MUTEX_INIT
+#  define THREAD GThread *
+#  define THREAD_EXIT(rval)   g_thread_exit(rval)
+#  define THREAD_JOIN(thread) g_thread_join(thread)
+#else
+#  define MUTEX pthread_mutex_t
+#  define MUTEX_LOCK(mutex)   pthread_mutex_lock(mutex)
+#  define MUTEX_UNLOCK(mutex) pthread_mutex_unlock(mutex)
+#  define MUTEX_INITIALIZER   PTHREAD_MUTEX_INITIALIZER
+#  define THREAD pthread_t
+#  define THREAD_EXIT(rval)   pthread_exit(rval)
+#  define THREAD_JOIN(thread) pthread_join(thread,NULL)
+#endif
+
 extern gboolean opened;		/* XMMS-crossfade is opened */
 extern gboolean output_opened;	/* the output plugin is opened */
 extern gint     output_offset;
