@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2007  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -100,9 +100,9 @@ static gint dsp_fragment_size    =  0;
 static gint dsp_fragment_utime   =  0;
 static gboolean dsp_select_works = FALSE;
 
-static pthread_mutex_t buffer_mutex;
-static pthread_t       buffer_thread;
-static gboolean        buffer_thread_finished;
+static MUTEX buffer_mutex = MUTEX_INITIALIZER;
+static pthread_t buffer_thread;
+static gboolean  buffer_thread_finished;
 
 static format_t format;
 
@@ -450,9 +450,6 @@ oss_open_audio(AFormat fmt, int rate, int nch)
 	stopped = FALSE;
 	paused  = FALSE;
 
-	/* create buffer mutex (never fails) */
-	pthread_mutex_init(&buffer_mutex, NULL);
-
 	/* create and run buffer thread (detached) */
 	buffer_thread_finished = FALSE;
 	pthread_attr_init(&attr);
@@ -494,7 +491,7 @@ oss_write_audio(void *ptr, int length)
 	}
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* print warning on buffer overrun */
 	free = buffer_size - buffer_used;
@@ -532,7 +529,7 @@ oss_write_audio(void *ptr, int length)
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 }
 
 
@@ -550,7 +547,7 @@ buffer_thread_f(void *arg)
 	DEBUG(("[xfade-oss]\n"));
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	while (dsp_fd != -1)
 	{
@@ -562,17 +559,17 @@ buffer_thread_f(void *arg)
 			FD_ZERO(&set);
 			FD_SET(dsp_fd, &set);
 
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			sel = select(dsp_fd + 1, NULL, &set, NULL, &tv);
-			pthread_mutex_lock(&buffer_mutex);
+			MUTEX_LOCK(&buffer_mutex);
 
 			if (sel == -1)
 			{
 				PERROR("[xfade-oss] buffer_thread_f: select()");
 #if 1
-				pthread_mutex_unlock(&buffer_mutex);
+				MUTEX_UNLOCK(&buffer_mutex);
 				xmms_usleep(dsp_fragment_utime);
-				pthread_mutex_lock(&buffer_mutex);
+				MUTEX_LOCK(&buffer_mutex);
 				continue;
 #else
 				break;
@@ -583,9 +580,9 @@ buffer_thread_f(void *arg)
 		}
 		else
 		{
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			xmms_usleep(dsp_fragment_utime);
-			pthread_mutex_lock(&buffer_mutex);
+			MUTEX_LOCK(&buffer_mutex);
 		}
 
 		/* get free space in device output buffer */
@@ -646,14 +643,14 @@ buffer_thread_f(void *arg)
 			/* HACK: Force yielding. This is _vital_ in realtime mode.
 			 *       Without it this thread would eat up all CPU time polling
 			 *       the audio device with select(), having no samples to write. */
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			xmms_usleep(dsp_fragment_utime / 4);
-			pthread_mutex_lock(&buffer_mutex);
+			MUTEX_LOCK(&buffer_mutex);
 		}
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	DEBUG(("[xfade-oss] buffer_thread_f: thread finished\n"));
 	buffer_thread_finished = TRUE;
@@ -667,7 +664,7 @@ void
 oss_close_audio()
 {
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* free buffer */
 	g_free(buffer_data);
@@ -681,7 +678,7 @@ oss_close_audio()
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	/* wait for buffer thread to terminate */
 #ifdef USE_PTHREAD_JOIN
@@ -692,10 +689,6 @@ oss_close_audio()
 		xmms_usleep(10000);
 #endif		
 
-	/* destroy mutex */
-	if (pthread_mutex_destroy(&buffer_mutex))
-		PERROR("[xfade-oss] close_audio: pthread_mutex_destroy()");
-
 	/* free effect private data */
 	effect_free(&effect_context);
 }
@@ -704,7 +697,7 @@ void
 oss_flush(int time)
 {
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	/* update streampos with new stream position (input format size) */
 	streampos = (gint64) time *format.bps / 1000;
@@ -719,7 +712,7 @@ oss_flush(int time)
 	buffer_preload  = buffer_preload_size;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 }
 
 void
@@ -729,7 +722,7 @@ oss_pause(short p)
 	gint rewind;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	paused = p;
 	if (paused)
@@ -737,7 +730,7 @@ oss_pause(short p)
 		/* how much data is left in the device buffer? */
 		if ((dsp_fd == -1) || (ioctl(dsp_fd, SNDCTL_DSP_GETOSPACE, &info) == -1))
 		{
-			pthread_mutex_unlock(&buffer_mutex);
+			MUTEX_UNLOCK(&buffer_mutex);
 			return;
 		}
 		rewind = dsp_buffer_size - info.bytes;
@@ -769,7 +762,7 @@ oss_pause(short p)
 	}
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 }
 
 gint
@@ -778,7 +771,7 @@ oss_buffer_free()
 	gint size, free;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	if (paused)
 		size = buffer_size;
@@ -790,7 +783,7 @@ oss_buffer_free()
 		free = 0;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	return free;
 }
@@ -803,11 +796,11 @@ oss_buffer_playing()
 	gboolean playing;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	if ((dsp_fd == -1) || (ioctl(dsp_fd, SNDCTL_DSP_GETOSPACE, &info) == -1))
 	{
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		return 0;
 	}
 
@@ -815,7 +808,7 @@ oss_buffer_playing()
 	playing = (buffer_used > 0) || (dsp_used >= 3 * dsp_fragment_size);
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	return playing;
 }
@@ -827,11 +820,11 @@ oss_output_time()
 	gint time;
 
 	/* lock buffer */
-	pthread_mutex_lock(&buffer_mutex);
+	MUTEX_LOCK(&buffer_mutex);
 
 	if ((dsp_fd == -1) || (ioctl(dsp_fd, SNDCTL_DSP_GETOSPACE, &info) == -1))
 	{
-		pthread_mutex_unlock(&buffer_mutex);
+		MUTEX_UNLOCK(&buffer_mutex);
 		return 0;
 	}
 
@@ -840,7 +833,7 @@ oss_output_time()
 		time = 0;
 
 	/* unlock buffer */
-	pthread_mutex_unlock(&buffer_mutex);
+	MUTEX_UNLOCK(&buffer_mutex);
 
 	return time;
 }
